# Generated by Django 3.2.9 on 2021-11-01 12:34

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("api", "0002_template_group"),
    ]

    operations = [
        migrations.AddField(
            model_name="template",
            name="meta",
            field=models.JSONField(default=dict),
        ),
    ]
